ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                         c
c      p p --> gamma gamma l nu                           c
c                                                         c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c Original programme by O. J. P. Eboli and S. M. Lietti of 
c The Instituto de Fisica Teorica, written by  S. M. Lietti
c
c Adapted by Paul Bell (Manchester) and Dusan Reljic (Belgrade)
c -pp events with correct quark mixing
c -unweighting of events with correct quark labels
c -assigning of intermediate W information (optional)
c -Les Houches accord output
c -assigning of ele or muon masses
c

! Main Program
      implicit none
      integer i
      real*8 dsigma, sigma, sd, chi2
           
! Vegas variables
      integer ncall, itmax, nprn, ndev
      real*8 xl(10), xu(10), acc
      common/bveg1/ncall, itmax, nprn, ndev, xl, xu, acc
      integer ndim
      parameter( ndim = 10)

! LHC energy 
      real*8       rs
      common/accel/rs

! HBOOK
      integer ncell
      logical hist
      common/dist/ncell,hist
      integer hmemor
      common/pawc/hmemor(1000000)
      common/maxwgt/wgt_max, wgt_maxnew, count
      real*8 wgt_max, wgt_maxnew 
      integer count(20)

! ANOMALOUS VARIABLES (definition in subroutines_DHelas_WWAA/wmwpaa_ano.f)
      real*8      fano(11)
      common/fano/fano
      real*8 gw2, sw2, cw2, sw, cw, czw
      real*8 k0, kc

c Kinematics cuts
      integer ncall_rd
      real*8  maxwgt_rd
      real*8 k0_inp, kc_inp
      real*8 
     +min_pt_a1,  min_pt_l, min_pt_n,
     +lim_eta_a, lim_eta_l,
     +min_dr_al, min_dr_aa, min_pt_a2
      common/cuts/
     +min_pt_a1,  min_pt_l, min_pt_n,
     +lim_eta_a, lim_eta_l,
     +min_dr_al, min_dr_aa, min_pt_a2

c Random number stuff
      integer startseed
      common/random/startseed
      integer timeArray(3)
      real rand

c choice of pdf
      CHARACTER*20 PARM(20)
      DOUBLE PRECISION VAL(20)
      COMMON /PDF/ IPTYPE,IGROUP,ISET
      INTEGER      IPTYPE,IGROUP,ISET

c ‘User Process’ Run Common Block
      integer MAXPUP
      parameter ( MAXPUP=100 )
      integer IDBMUP, PDFGUP, PDFSUP, IDWTUP, NPRUP, LPRUP
      double precision EBMUP, XSECUP, XERRUP, XMAXUP
      common /HEPRUP/ IDBMUP(2), EBMUP(2), PDFGUP(2), PDFSUP(2),
     + IDWTUP, NPRUP, XSECUP(MAXPUP), XERRUP(MAXPUP),
     + XMAXUP(MAXPUP), LPRUP(MAXPUP)

c ‘User Process’ Event Common Block
      integer MAXNUP
      parameter ( MAXNUP=500 )
      integer NUP, IDPRUP, IDUP, ISTUP, MOTHUP, ICOLUP
      double precision XWGTUP, SCALUP, AQEDUP, AQCDUP,
     + PUP, VTIMUP, SPINUP
      common /HEPEUP/ NUP, IDPRUP, XWGTUP, SCALUP, AQEDUP, AQCDUP,
     + IDUP(MAXNUP), ISTUP(MAXNUP), MOTHUP(2,MAXNUP),
     + ICOLUP(2,MAXNUP), PUP(5,MAXNUP), VTIMUP(MAXNUP),
     + SPINUP(MAXNUP)

c W and ele/muon information flags
      integer wflag
      real elmuflag
      common/wfl/wflag, elmuflag 

c actual routine
      external dsigma

c set PDFs cteq 5l
      PARM(1)='NPTYPE'
      VAL(1) = 1
      PARM(2)='NGROUP'
      VAL(2) = 4
      PARM(3)='NSET'
      VAL(3) = 46
      CALL PDFSET(PARM,VAL)

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
! BEGIN CODE
!
*.....vegas inputs
      ndev = 6
      nprn = 0
            
*.....LHC energy in GeV 
      rs=14.d3

c do or do not use w info
      wflag = 0;
c 50% muons
      elmuflag = 0.5

c Open LHout.txt file for HEPEUP
      open(80,file=
     +'/afs/cern.ch/user/p/paulb/output/LH_event.txt')

c Open LHout2.txt file for HEPRUP
      open(90,file=
     +'/afs/cern.ch/user/p/paulb/output/LH_init.txt')

c Open wmass.txt file for W mass
      open(20,file=
     +'/afs/cern.ch/user/p/paulb/output/wmass.txt')

c Read in cards file for anom couplings and cuts
        open(40,file=
     +'/afs/cern.ch/user/p/pbell/W2PHO/input/cards.dat')
      read(40,*)
      read(40,*) ncall_rd
      read(40,*)
      read(40,*) wgt_max
      read(40,*)
      read(40,*) k0_inp
      read(40,*)
      read(40,*) kc_inp
      read(40,*)
      read(40,*) min_pt_a1
      read(40,*)
      read(40,*) min_pt_a2
      read(40,*)
      read(40,*) min_pt_l
      read(40,*)
      read(40,*) min_pt_n
      read(40,*)
      read(40,*) lim_eta_a
      read(40,*)
      read(40,*) lim_eta_l
      read(40,*)
      read(40,*) min_dr_aa
      read(40,*)
      read(40,*) min_dr_al

      write(*,*) "ncall/wgt_max              ", ncall_rd, wgt_max
      write(*,*) "k0:                        ", k0_inp
      write(*,*) "kc:                        ", kc_inp
      write(*,*) "Min pt pho1:               " ,min_pt_a1
      write(*,*) "Min pt pho2:               ", min_pt_a2
      write(*,*) "Min pt lepton:             ", min_pt_l
      write(*,*) "Min pt_neutrino:           ", min_pt_n
      write(*,*) "Max eta_ epton:            ", lim_eta_l
      write(*,*) "Max eta photon:            ", lim_eta_a
      write(*,*) "Min pho-pho separation:    ", min_dr_al
      write(*,*) "Min pho-lepton separation: ", min_dr_aa

*.....Initializing data used by Helas subroutines
      call initialize

*.....Initializing Anomalous variables     
      fano( 1)= 0.d0          ! beta_W0a
      fano( 2)= 0.d0          ! beta_Wca
      fano( 3)= 0.d0          ! beta_W0Z  
      fano( 4)= 0.d0          ! beta_WcZ  
      fano( 5)= 0.d0          ! beta_W1Z  
      fano( 6)= 0.d0          ! beta_W2Z  
      fano( 7)= 0.d0          ! beta_W3Z  
      fano( 8)= 0.d0          ! beta_Z0a  
      fano( 9)= 0.d0          ! beta_Zca  
      fano(10)= 0.d0          ! beta_Z0Z  
      fano(11)= 0.d0          ! beta_ZcZ

      gw2 = 0.424557907d0
      sw2 = .23124d0
      cw2 = 1.d0-sw2
      sw  = dsqrt(sw2)
      cw  = dsqrt(cw2)
      czw = (cw2-sw2)/(2*cw*sw)
c
c Only fano(1) and (2) are used as the others refer to
c helas routines for vertices other than wwgg. 
c fano(1) and (2) are beta_0 and beta_c in the Lietti paper 
c and the same as a0 and ac.
c
      fano( 1)= k0_inp
      fano( 3)= -k0 * 4.d0 * gw2 * (sw/cw) ! beta_W0Z  
      fano( 8)=  k0 * 4.d0 * gw2           ! beta_Z0a   
      fano(10)= -k0 * 4.d0 * gw2 * (sw/cw) ! beta_Z0Z

      fano( 2)= kc_inp
      fano( 4)= -kc * 4.d0 * gw2 * (sw/cw) ! beta_WcZ  
      fano( 9)=  kc * 4.d0 * gw2           ! beta_Zca   
      fano(11)= -kc * 4.d0 * gw2 * (sw/cw) ! beta_ZcZ

c fpr Les Houches
      AQEDUP=1/127.9
      AQCDUP=-1

* setup random selection of events with wgt/wgtmax
* wgt max has to be found from a large run and stored somewhere

      call itime(timeArray)
      write(*,*) 'rand init:',timeArray(1),timeArray(2),timeArray(3),
     +timeArray(1)+timeArray(2)+timeArray(3)

      startseed=rand(timeArray(1)+timeArray(2)+timeArray(3))

c     reset counters
      count(1)  = 0
      count(2)  = 0
      count(3)  = 0
      count(4)  = 0
      count(5)  = 0
      count(6)  = 0
      count(7)  = 0
      count(8)  = 0
      count(9)  = 0
      count(10) = 0
      count(11) = 0
      count(12) = 0
      count(13) = 0
      count(14) = 0
      count(15) = 0
      count(16) = 0
      count(17) = 0
      count(18) = 0
      count(19) = 0
      count(20) = 0

*.....Initializing the distributions
      call hlimit(100000)
*......call hbook1(n_hist,'name var',n_bins,lim_inf,lim_sup,0.)
      call hbook1(25 , 'eta(a1)'        , 20 , -3.  , 3.    , 0.)  
      call hbook1(26 , 'eta(a2)'        , 20 , -3.  , 3.    , 0.) 
      call hbook1(27 , 'eta(ele)'       , 20 , -3.  , 3.    , 0.)  
      call hbook1(3 , 'pt(a1)'         , 20 ,  0.  , 200. , 0.)  
      call hbook1(4 , 'pt(a2)'         , 20 ,  0.  , 200. , 0.) 
      call hbook1(5 , 'm(a1a2)'        , 20 ,  0.  , 500. , 0.) 
      call hbook1(6 , 'dr(a1a2)'       , 20 ,  0.  , 6.    , 0.)  
      call hbook1(61 , 'dr(la1)'       , 20 ,  0.  , 6.    , 0.)  
      call hbook1(62 , 'dr(la2)'       , 20 ,  0.  , 6.    , 0.)  
      call hbook1(7 , 'phi(a1)'        , 20 , -1.6 , 1.6   , 0.)     
      call hbook1(8 , 'phi(a2)'        , 20 , -1.6 , 1.6   , 0.)  
      call hbook1(9 , 'cos_theta(a1)'  , 20 , -1.  , 1.    , 0.)   
      call hbook1(10, 'cos_theta(a2)'  , 20 , -1.  , 1.    , 0.)  
      call hbook1(11, 'm W'            , 50 ,  0.  , 200.  , 0.)
      call hbook1(12, 'mt W'           , 50 ,  0.  , 200.  , 0.)
      call hbook1(13, 'weight'         , 50 ,  0.  , 1.e-10, 0.)
      call hbook1(14, 'proby'          , 50 ,  0.  , 1.e-7,  0.)
      call hbook1(15, 'random'         , 50 ,  0.  , 1.,     0.)

      call hbook1(1025  , 'eta(a1)'      , 20 , -3.  , 3.    , 0.)  
      call hbook1(1026  , 'eta(a2)'      , 20 , -3.  , 3.    , 0.) 
      call hbook1(1027  , 'eta(ele)'     , 20 , -3.  , 3.    , 0.) 
      call hbook1(103 , 'pt(a1)'         , 20 ,  0.  , 200. , 0.)  
      call hbook1(104 , 'pt(a2)'         , 20 ,  0.  , 200. , 0.) 
      call hbook1(105 , 'm(a1a2)'        , 20 ,  0.  , 500. , 0.) 
      call hbook1(106 , 'dr(a1a2)'       , 20 ,  0.  , 6.    , 0.)  
      call hbook1(1061 , 'dr(la1)'       , 20 ,  0.  , 6.    , 0.)  
      call hbook1(1062 , 'dr(la2)'       , 20 ,  0.  , 6.    , 0.)  
      call hbook1(107 , 'phi(a1)'        , 20 , -1.6 , 1.6   , 0.)     
      call hbook1(108 , 'phi(a2)'        , 20 , -1.6 , 1.6   , 0.)  
      call hbook1(109 , 'cos_theta(a1)'  , 20 , -1.  , 1.    , 0.)   
      call hbook1(1010, 'cos_theta(a2)'  , 20 , -1.  , 1.    , 0.)  
      call hbook1(1011, 'm W'            , 50 ,  0.  , 200.  , 0.)
      call hbook1(1012, 'mt W'           , 50 ,  0.  , 200.  , 0.)

*.....initializing the grid (Vegas)  
      hist = .false. 
      itmax = 10
c     ncall = 10000 as used in samples
      ncall = 10000
      call vegas(ndim, dsigma, sigma, sd, chi2)

*....the real integration (Vegas)
      hist = .true.
      itmax = 40
      ncall = ncall_rd

c recall - more events, so smaller weight, so fewer selected
c but more anyway so same number selected in the end!
c used 4000000 with noff so this is similar indeed      
     
      call vegas1(ndim, dsigma, sigma, sd, chi2)

*.....printing the results
      open(1,name=
     +'/afs/cern.ch/user/p/paulb/output/summary.dat')
      write(1,*)'Cross-section(fb) = ',sigma*1000
      write(1,*)'Error(pb) = ',sd
      write(1,*)'Chi2 = ',chi2
      write(1,*)'Final max weight ', wgt_maxnew, wgt_max
      write(1,*)'Unweighted events ', count
      write(1,*)"alarm events", count(20)
      write(1,*)"total events", count(19)
      close(1)

c LH summary read from the main program
      XSECUP(1)=sigma
      XERRUP(1)=sd
      XMAXUP(1)=-100      
      LPRUP(1)=999

c  write HEPRUP
      write(90,*) IDBMUP(1), IDBMUP(2), EBMUP(1), EBMUP(2),
     +            PDFGUP(1), PDFGUP(2), PDFSUP(1), PDFSUP(2),
     +            IDWTUP, NPRUP, XSECUP(1), XERRUP(1),
     +            XMAXUP(1), LPRUP(1)

*.....distributions output
      call hrput(0,
     +'/afs/cern.ch/user/p/paulb/output/pp_wgg.hbook','n')

      end

!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

      function dsigma(x,wgt)

      implicit none

! GENERAL VARIABLES
      integer i      
      real*8 pi,conv
      parameter(pi =  3.141592654d0)
      parameter(conv =  3.8937966d8) ! 1/GeV^2 -> pb
      real*8 dot_d

! VEGAS
      real*8 x(10),xx(10),wgt,dsigma
      real*8 x1,x2,taumax,taumin,tau,y,ym,scale

! STRUCTURE FUNCTIONS AND AMPLITUDE
      real*8 upv1,dnv1,usea1,dsea1,str1,chm1,bot1,top1,gl1
      real*8 upv2,dnv2,usea2,dsea2,str2,chm2,bot2,top2,gl2

c
      real*8 quarks_dub, quarks_ubd
      real*8 quarks_scb, quarks_cbs
      real*8 quarks_dcb, quarks_cbd
      real*8 quarks_sub, quarks_ubs
c
      real*8 quarks_udb, quarks_dbu
      real*8 quarks_csb, quarks_sbc
      real*8 quarks_usb, quarks_sbu
      real*8 quarks_cdb, quarks_dbc

      real*8 mxe_d_ub,mxe_ub_d
      real*8 mxe_u_db,mxe_db_u

      real*8 dsigma_dub, dsigma_ubd
      real*8 dsigma_scb, dsigma_cbs
      real*8 dsigma_dcb, dsigma_cbd
      real*8 dsigma_sub, dsigma_ubs
c
      real*8 dsigma_udb, dsigma_dbu
      real*8 dsigma_csb, dsigma_sbc
      real*8 dsigma_usb, dsigma_sbu
      real*8 dsigma_cdb, dsigma_dbc

 
      real*8 amplsq_dub, amplsq_ubd
      real*8 amplsq_scb, amplsq_cbs
      real*8 amplsq_dcb, amplsq_cbd
      real*8 amplsq_sub, amplsq_ubs
c
      real*8 amplsq_udb, amplsq_dbu
      real*8 amplsq_csb, amplsq_sbc
      real*8 amplsq_usb, amplsq_sbu
      real*8 amplsq_cdb, amplsq_dbc

      real*8 amplsq

      real*8 sudb_aaepve_ano,sdub_aaemve_ano

! KINEMATICS      
      real*8 et_n , et_e , et_a1 , et_a2
      real*8 eta_n, eta_e, eta_a1, eta_a2 
      real*8 pt_n , pt_e , pt_a1 , pt_a2
      real*8 cos_n, cos_e, cos_a1, cos_a2
      real*8 phi_n, phi_e, phi_a1, phi_a2
      real*8 im_aa, im_ea1, im_ea2
      real*8 dr_aa, dr_ea1, dr_ea2
      real*8 mt_w, m_w
      real*8 pen(0:3), ptenu
      real*8 DENOM, TEMP, DELTA_PHI

! MOMENTUM AND PHASE SPACE
      real*8 s,uni
      real*8       rs
      common/accel/rs
      real*8 rshat, shat
      real*8 plab(0:3),paux(0:3)
      real*8 pa(0:3),pb(0:3),p1(0:3),p2(0:3),p3(0:3),p4(0:3)
      real*8 ma, mb , m12max, m12min, m34max, m34min
      real*8      m1, m2, m3, m4
      common/mass/m1, m2, m3, m4
      real*8         jacobian, flux 
      common/momenta/jacobian, flux 

! HBOOK
      integer ncell
      logical hist
      common/dist/ncell,hist
      integer hmemor
      common/pawc/hmemor(1000000)
      real weight_eta,weight_pt,weight_im
      real weight_dr,weight_phi,weight_cos
      real weight_dr_ea1, weight_dr_ea2

      real weight_mt_w, weight_m_w
      real h_mt_w, h_m_w
      real h_eta_a1 , h_eta_a2, h_eta_e 
      real h_pt_a1  , h_pt_a2  
      real h_im_aa
      real h_dr_aa
      real h_dr_ea1
      real h_dr_ea2
      real h_phi_a1,h_phi_a2
      real h_cos_a1,h_cos_a2  

      integer un_wgt
      common/maxwgt/wgt_max, wgt_maxnew, count
      real*8 wgt_max, wgt_maxnew 
      integer count(20)
      
      real*8 proby, Rvec(1), R, Rq

      real*8 proby_dub, proby_ubd
      real*8 proby_scb, proby_cbs
      real*8 proby_dcb, proby_cbd
      real*8 proby_sub, proby_ubs

      real*8 proby_udb, proby_dbu
      real*8 proby_csb, proby_sbc
      real*8 proby_usb, proby_sbu
      real*8 proby_cdb, proby_dbc
      real*8 proby_tot

      integer startseed
      common/random/startseed
      real rand

! ANOMALOUS VARIABLES (definition in subroutines_DHelas_eWAA/wmwpaa_ano.f)
      real*8     gano(11)
      common/ano/gano 
      real*8      fano(11)
      common/fano/fano

c pjb kinematics cuts

      real*8 k0_inp, kc_inp
      real*8 
     +min_pt_a1,  min_pt_l, min_pt_n, 
     +lim_eta_a, lim_eta_l,
     +min_dr_al, min_dr_aa, min_pt_a2
      common/cuts/
     +min_pt_a1,  min_pt_l, min_pt_n,
     +lim_eta_a, lim_eta_l,
     +min_dr_al, min_dr_aa, min_pt_a2

c 'User Process' Run Common Block
      integer MAXPUP
      parameter ( MAXPUP=100 )
      integer IDBMUP, PDFGUP, PDFSUP, IDWTUP, NPRUP, LPRUP
      double precision EBMUP, XSECUP, XERRUP, XMAXUP
      common /HEPRUP/ IDBMUP(2), EBMUP(2), PDFGUP(2), PDFSUP(2),
     + IDWTUP, NPRUP, XSECUP(MAXPUP), XERRUP(MAXPUP),
     + XMAXUP(MAXPUP), LPRUP(MAXPUP)


c 'User Process' Event Common Block
      integer MAXNUP
      parameter ( MAXNUP=500 )
      integer NUP, IDPRUP, IDUP, ISTUP, MOTHUP, ICOLUP
      double precision XWGTUP, SCALUP, AQEDUP, AQCDUP,
     + PUP, VTIMUP, SPINUP
      common /HEPEUP/ NUP, IDPRUP, XWGTUP, SCALUP, AQEDUP, AQCDUP,
     + IDUP(MAXNUP), ISTUP(MAXNUP), MOTHUP(2,MAXNUP),
     + ICOLUP(2,MAXNUP), PUP(5,MAXNUP), VTIMUP(MAXNUP),
     + SPINUP(MAXNUP)

c W flag, and el/mu fraction
      integer wflag
      real elmuflag
      common/wfl/wflag, elmuflag
      integer event

c wflag  variables for finding the type of event based on w mass
      double precision MW1, MW2, MW3, MW4
      integer evtyp, MWMASSTYPE
      common /WMASS/ MW1, MW2, MW3, MW4

c b parameter for fixing the lepton mass
      double precision beta
      double precision elemass, mumass

c variables for labeling of electron/muon
      real elmurand
      integer lepidup
      integer neutidup

! BEGIN CODE
!  
*....Masses for the particles in GeV
       ma=0.d0     ! initial quark mass
       mb=0.d0     ! initial anti-quark mass
       m1=0.d0     ! final photon mass
       m2=0.d0     ! final photon mass
       m3=0.d0     ! final electron mass as in initialize.f
       m4=0.d0     ! final neutrino as in initialize.f

c electron and muon masses (in GeV)
      elemass = 0.000511
      mumass  = 0.105658369 

C START WRITING HEPRUP COMMON BLOCK
c define incoming protons
      IDBMUP(1)=2212
      IDBMUP(2)=2212
      EBMUP(1)=7.d3
      EBMUP(2)=7.d3

c check this values - same as set at the top
      PDFGUP(1)=4
      PDFGUP(2)=4
      PDFSUP(1)=46
      PDFSUP(2)=46

c all events are already unweighted, only one process
      IDWTUP=3
      NPRUP=1

c START WRITING HEPEUP COMMON BLOCK
c if W there are 7 particles, if not there are 6
      if(wflag .EQ. 0) then
         NUP=6
      else
         NUP=7
      endif

c process id and cross section source
      IDPRUP=999
      XWGTUP=1

*....Momentum (initial particles) 
       taumax=1.d0
       taumin=(m1+m2+m3)**2/rs/rs
       tau=taumin+(taumax-taumin)*x(1)
       ym=-1./2.*log(tau)    
       y=-ym+2.*ym*x(2)  
       x1=sqrt(tau)*exp(y)
       x2=sqrt(tau)*exp(-y)
       scale=rs*sqrt(tau)

c read from the main program
      SCALUP=scale

!pa(0:3) initial quark momentum
       pa(0)=(rs/2.d0)*x1
       pa(1)=0.d0
       pa(2)=0.d0
       pa(3)=pa(0)
!pb(0:3) initial anti-quark momentum
       pb(0)=(rs/2.d0)*x2
       pb(1)=0.d0
       pb(2)=0.d0
       pb(3)=-pb(0)
       
*....Vegas 
       do i=1,8
        xx(i)=x(i+2)  
       end do 
  
*....Momentum (final particles)
       
       do i = 0,3
          plab(i)=pa(i)+pb(i)
       end do
       shat  = dot_d(plab,plab)
       rshat = dsqrt(shat)
       m12min = m1+m2
       m34min = m3+m4
       m12max = rshat - m34min
       m34max = rshat - m12min
        
!p1(0:3)  final a1 photon momentum
!p2(0:3)  final a2 photon momentum
!p3(0:3)  final electron momentum 
!p4(0:3)  final neutrino momentum 
       call phase_4(xx,m12max,m12min,m34max,m34min,plab,p1,p2,p3,p4)

*....Ordering photon energy [E_a1 > E_a2 => p1(0) > p2(0)]
       if (p2(0).gt.p1(0)) then
          do i=0,3
             paux(i) = p1(i)
             p1(i)   = p2(i)
             p2(i)   = paux(i)
          end do
       end if

*....KINEMATICAL CUTS
*....ANGLES
	cos_a1 = p1(3)/dsqrt(p1(1)**2+p1(2)**2+p1(3)**2)
	cos_a2 = p2(3)/dsqrt(p2(1)**2+p2(2)**2+p2(3)**2)
	cos_e  = p3(3)/dsqrt(p3(1)**2+p3(2)**2+p3(3)**2)
	cos_n  = p4(3)/dsqrt(p4(1)**2+p4(2)**2+p4(3)**2) 

        phi_a1 = datan(p1(2)/p1(1))
        phi_a2 = datan(p2(2)/p2(1))
        phi_e  = datan(p3(2)/p3(1))
        phi_n  = datan(p4(2)/p4(1))
	      
*....TRANSVERSE MOMENTUM
        pt_a1 = dsqrt(p1(2)**2+p1(1)**2)
        pt_a2 = dsqrt(p2(2)**2+p2(1)**2)        
        pt_e  = dsqrt(p3(2)**2+p3(1)**2)        
        pt_n  = dsqrt(p4(2)**2+p4(1)**2)

        if(pt_a1 .lt. min_pt_a1 .or.
     %     pt_a2 .lt. min_pt_a2 .or.
     %     pt_e  .lt. min_pt_l .or.
     %     pt_n  .lt. min_pt_n) then
           dsigma= 0.d0
           return
        endif

*....INVARIANT MASSES
        im_aa  = dsqrt((p1(0)+p2(0))**2 - (p1(1)+p2(1))**2
     &               - (p1(2)+p2(2))**2 - (p1(3)+p2(3))**2) 
        im_ea1 = dsqrt((p1(0)+p3(0))**2 - (p1(1)+p3(1))**2
     &               - (p1(2)+p3(2))**2 - (p1(3)+p3(3))**2) 
        im_ea2 = dsqrt((p2(0)+p3(0))**2 - (p2(1)+p3(1))**2
     &               - (p2(2)+p3(2))**2 - (p2(3)+p3(3))**2)
        
c        if(im_aa .lt.   400.d0 .or.
c     %     im_aa .gt. 2500.d0) then
c           dsigma= 0.d0
c           return
c        endif
        
*....TRANSVERSE ENERGY
        et_a1 = p1(0)*dsqrt(1.d0-cos_a1**2)
        et_a2 = p2(0)*dsqrt(1.d0-cos_a2**2)                         
        et_e  = p3(0)*dsqrt(1.d0-cos_e**2)                        
c pjb        et_n  = p3(0)*dsqrt(1.d0-cos_n**2)
        et_n  = p4(0)*dsqrt(1.d0-cos_n**2)

        do i = 0,3
           pen(i)=p3(i)+p4(i)
        enddo
        ptenu = dsqrt(pen(1)**2+pen(2)**2)
        mt_w = dsqrt((pt_e + pt_n)**2 - ptenu**2)
        
c Optional cut on transverse mass
c        if (mt_w.lt.65.0  ) then
c           dsigma= 0.d0
c           return
c        end if
c        if (mt_w.gt.100.0  ) then
c           dsigma= 0.d0
c           return
c        end if

c check what e and u recon

        m_w = dsqrt((p3(0)+p4(0))**2
     &      -(p3(1)+p4(1))**2
     &      -(p3(2)+p4(2))**2
     &      -(p3(3)+p4(3))**2)

*....PSEUDORAPIDITY
        eta_a1 = -0.5d0*log((1-cos_a1)/(1+cos_a1))        
        eta_a2 = -0.5d0*log((1-cos_a2)/(1+cos_a2))
        eta_e  = -0.5d0*log((1-cos_e)/(1+cos_e))
        eta_n  = -0.5d0*log((1-cos_n)/(1+cos_n))

        if (dabs(eta_a1) .gt. lim_eta_a .or.
     %      dabs(eta_a2) .gt. lim_eta_a .or.
     %      dabs(eta_e)  .gt. lim_eta_l ) then
           dsigma= 0.d0
           return
        end if

c separation of photons
        DENOM = SQRT(p1(1)**2 + p1(2)**2)
     +        * SQRT(p2(1)**2 + p2(2)**2)

        TEMP =
     +       MAX(-0.99999999D0, (p1(1)*p2(1) + p1(2)*p2(2)) /DENOM)
        TEMP = MIN( 0.99999999D0, TEMP)
        DELTA_PHI = ACOS(TEMP)
        dr_aa  = sqrt(delta_phi**2+(eta_a1-eta_a2)**2)

c separation of photon and lepton 1
        DENOM = SQRT(p1(1)**2 + p1(2)**2)
     +        * SQRT(p3(1)**2 + p3(2)**2)

        TEMP =
     +       MAX(-0.99999999D0, (p1(1)*p3(1) + p1(2)*p3(2)) /DENOM)
        TEMP = MIN( 0.99999999D0, TEMP)
        DELTA_PHI = ACOS(TEMP)
        dr_ea1  = sqrt(delta_phi**2+(eta_a1-eta_e)**2)

c separation of photon and lepton 2
        DENOM = SQRT(p3(1)**2 + p3(2)**2)
     +        * SQRT(p2(1)**2 + p2(2)**2)

        TEMP =
     +       MAX(-0.99999999D0, (p3(1)*p2(1) + p3(2)*p2(2)) /DENOM)
        TEMP = MIN( 0.99999999D0, TEMP)
        DELTA_PHI = ACOS(TEMP)
        dr_ea2  = sqrt(delta_phi**2+(eta_e-eta_a2)**2)

        if (dr_aa  .lt. min_dr_aa .or.
     %      dr_ea1 .lt. min_dr_al .or.
     %      dr_ea2 .lt. min_dr_al) then
           dsigma= 0.d0
           return 
        endif   
         
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
*.... STRUCTURE FUNCTIONS
         
       if(x1.le.1.d-4 .or. x2.le.1.d-4 .or. 
     %    scale.le.dsqrt(10.d0)) then 
           dsigma= 0.d0
           return
       endif
      
       call structm(x1,scale,upv1,dnv1,usea1,
     %              dsea1,str1,chm1,bot1,top1,gl1)
       
       call structm(x2,scale,upv2,dnv2,usea2,
     %              dsea2,str2,chm2,bot2,top2,gl2)

cccccccccccccc
c d ub part W-
c
c down from proton 1, up from proton 2
c d ub
       quarks_dub  = ((dnv1+dsea1)*usea2)/tau
c s cb
       quarks_scb  = (str1*chm2)/tau

c d cb
       quarks_dcb  = ((dnv1+dsea1)*chm2)/tau
c s ub
       quarks_sub  = (str1*usea2)/tau


c up from proton 1, down from proton 2
c ub d
       quarks_ubd  = (usea1*(dnv2+dsea2))/tau
c cb s
       quarks_cbs  = (chm1*str2)/tau

c cb d
       quarks_cbd  = (chm1*(dnv2+dsea2))/tau
c ub s
       quarks_ubs  = (usea1*str2)/tau    
c
c
c
cccccccccccccc
c u db part W+
c
c up from proton 1, down from proton 2

       quarks_udb  = ((upv1+usea1)*dsea2)/tau

       quarks_csb = (chm1*str2)/tau

       quarks_usb = ((upv1+usea1)*str2)/tau

       quarks_cdb = (chm1*dsea2)/tau

c down from proton 1, up from proton 2

       quarks_dbu = (dsea1*(upv2+usea2))/tau

       quarks_sbc = (str1*chm2)/tau

       quarks_sbu = (str1*(upv2+usea2))/tau

       quarks_dbc = (dsea1*chm2)/tau

       
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
*....UNITARITY CONDITION (only for anomalous WWAA vertex)
c       uni=1/(1.d0+(im_aa/2.5d3)**2)**5
       uni=1.d0
       do i=1,3
          gano(i)=fano(i)*uni
       end do

*....AMPLITUDES

c d ub part W-

       mxe_d_ub = sdub_aaemve_ano(pa,pb,p1,p2,p3,p4)
       mxe_ub_d = sdub_aaemve_ano(pb,pa,p1,p2,p3,p4)

       amplsq_dub  = quarks_dub * mxe_d_ub
       amplsq_scb  = quarks_scb * mxe_d_ub
       amplsq_dcb  = quarks_dcb * mxe_d_ub
       amplsq_sub  = quarks_sub * mxe_d_ub

       amplsq_ubd  = quarks_ubd * mxe_ub_d
       amplsq_cbs  = quarks_cbs * mxe_ub_d
       amplsq_cbd  = quarks_cbd * mxe_ub_d
       amplsq_ubs  = quarks_ubs * mxe_ub_d
       
c u db part W+

       mxe_u_db = sudb_aaepve_ano(pa,pb,p1,p2,p3,p4)
       mxe_db_u = sudb_aaepve_ano(pb,pa,p1,p2,p3,p4)

       amplsq_udb  = quarks_udb * mxe_u_db
       amplsq_csb  = quarks_csb * mxe_u_db
       amplsq_usb  = quarks_usb * mxe_u_db
       amplsq_cdb  = quarks_cdb * mxe_u_db

       amplsq_dbu  = quarks_dbu * mxe_db_u
       amplsq_sbc  = quarks_sbc * mxe_db_u
       amplsq_sbu  = quarks_sbu * mxe_db_u
       amplsq_dbc  = quarks_dbc * mxe_db_u

c dsigmas separated 

       dsigma_dub  
     + = jacobian*conv/flux*(taumax-taumin)*2*ym*amplsq_dub
       dsigma_scb  
     + = jacobian*conv/flux*(taumax-taumin)*2*ym*amplsq_scb
       dsigma_dcb  
     + = jacobian*conv/flux*(taumax-taumin)*2*ym*amplsq_dcb
       dsigma_sub  
     + = jacobian*conv/flux*(taumax-taumin)*2*ym*amplsq_sub

       dsigma_ubd  
     + = jacobian*conv/flux*(taumax-taumin)*2*ym*amplsq_ubd
       dsigma_cbs  
     + = jacobian*conv/flux*(taumax-taumin)*2*ym*amplsq_cbs
       dsigma_cbd  
     + = jacobian*conv/flux*(taumax-taumin)*2*ym*amplsq_cbd
       dsigma_ubs  
     + = jacobian*conv/flux*(taumax-taumin)*2*ym*amplsq_ubs


       dsigma_udb 
     + = jacobian*conv/flux*(taumax-taumin)*2*ym*amplsq_udb
       dsigma_csb 
     + = jacobian*conv/flux*(taumax-taumin)*2*ym*amplsq_csb
       dsigma_usb 
     + = jacobian*conv/flux*(taumax-taumin)*2*ym*amplsq_usb
       dsigma_cdb 
     + = jacobian*conv/flux*(taumax-taumin)*2*ym*amplsq_cdb

       dsigma_dbu 
     + = jacobian*conv/flux*(taumax-taumin)*2*ym*amplsq_dbu
       dsigma_sbc 
     + = jacobian*conv/flux*(taumax-taumin)*2*ym*amplsq_sbc
       dsigma_sbu 
     + = jacobian*conv/flux*(taumax-taumin)*2*ym*amplsq_sbu
       dsigma_dbc 
     + = jacobian*conv/flux*(taumax-taumin)*2*ym*amplsq_dbc

        
c combined amplitude
       
       dsigma_dub = 0.941*dsigma_dub
       dsigma_ubd = 0.941*dsigma_ubd
       dsigma_scb = 0.941*dsigma_scb 
       dsigma_cbs = 0.941*dsigma_cbs
       dsigma_dcb = 0.048*dsigma_dcb
       dsigma_cbd = 0.048*dsigma_cbd
       dsigma_sub = 0.048*dsigma_sub
       dsigma_ubs = 0.048*dsigma_ubs
       dsigma_udb = 0.941*dsigma_udb
       dsigma_dbu = 0.941*dsigma_dbu
       dsigma_csb = 0.941*dsigma_csb
       dsigma_sbc = 0.941*dsigma_sbc
       dsigma_usb = 0.048*dsigma_usb 
       dsigma_sbu = 0.048*dsigma_sbu
       dsigma_cdb = 0.048*dsigma_cdb 
       dsigma_dbc = 0.048*dsigma_dbc

       dsigma= dsigma_dub  +  dsigma_ubd 
     +       + dsigma_scb  +  dsigma_cbs 
     +       + dsigma_dcb  +  dsigma_cbd 
     +       + dsigma_sub  +  dsigma_ubs 
     +       + dsigma_udb  +  dsigma_dbu 
     +       + dsigma_csb  +  dsigma_sbc 
     +       + dsigma_usb  +  dsigma_sbu 
     +       + dsigma_cdb  +  dsigma_dbc 
       
       if(hist) then

c Now unweight relative quark contributions
c quarks_dub_1 quarks_sc_1 etc are relative probabilities
       
c     plot weighted events
c     itmax had better be fixed at 40!
c     *........weight = dsigma * wgt / itmax / ((mim_sup - lim_if)/n_bins)
 100      weight_eta = dsigma * wgt / 40 / ((3. - (-3.))/20)
          weight_pt = dsigma * wgt / 40 / ((200. - 0.)/20) 
          weight_im = dsigma * wgt / 40 / ((500. - 0.)/20)
          weight_dr = dsigma * wgt / 40 / ((6. - 0.)/20) 
          weight_dr_ea1 = dsigma * wgt / 40 / ((6. - 0.)/20) 
          weight_dr_ea2 = dsigma * wgt / 40 / ((6. - 0.)/20) 
          weight_phi = dsigma * wgt / 40 / ((1.6 - (-1.6))/20)  
          weight_cos = dsigma * wgt / 40 / ((1. - (-1.))/20) 
          weight_m_w  = dsigma * wgt / 40 / ((200. - 0.)/50) 
          weight_mt_w = dsigma * wgt / 40 / ((200. - 0.)/50) 
          
          h_m_w  = m_w
          h_mt_w = mt_w
          h_eta_a1 = eta_a1  
          h_eta_a2 = eta_a2
          h_eta_e = eta_e
          h_pt_a1 = pt_a1  
          h_pt_a2 = pt_a2
          h_im_aa = im_aa
          h_dr_aa = dr_aa
          h_dr_ea1 = dr_ea1
          h_dr_ea2 = dr_ea2
          h_phi_a1 = phi_a1  
          h_phi_a2 = phi_a2 
          h_cos_a1 = cos_a1  
          h_cos_a2 = cos_a2
          call hf1(25,h_eta_a1,weight_eta)
          call hf1(26,h_eta_a2,weight_eta)
          call hf1(27,h_eta_e,weight_eta)
          call hf1(3,h_pt_a1,weight_pt)
          call hf1(4,h_pt_a2,weight_pt)
          call hf1(5,h_im_aa,weight_im)
          call hf1(6,h_dr_aa,weight_dr)
          call hf1(61,h_dr_ea1,weight_dr_ea1)
          call hf1(62,h_dr_ea2,weight_dr_ea2)
          call hf1(7,h_phi_a1,weight_phi)
          call hf1(8,h_phi_a2,weight_phi)
          call hf1(9,h_cos_a1,weight_cos)
          call hf1(10,h_cos_a2,weight_cos)
          call hf1(11,h_m_w,weight_m_w)
          call hf1(12,h_mt_w,weight_mt_w)
          call hf1(13,real((wgt*dsigma)/5.0),1.0)
          call hf1(14,real(proby),1.0)
          call hf1(15,real(R),1.0)
          call hf1(15,real(R),1.0)
          
c     now the UNWEIGHTING
c     want to select event with proby wgt/wgtmax
c     generate a random number R between 0 and 1 with a flat dist
c     if proby>R then take it,

c count total events
          count(19) = count(19) + 1

c see if max wgt exceeded
          if((wgt*dsigma).gt.wgt_max) then
             write(*,*) "ALARM: max wgt exceeded", wgt*dsigma
             count(20) = count(20) + 1
             wgt_maxnew = wgt*dsigma
          endif

          proby = (wgt*dsigma)/wgt_max
          R=rand(startseed)
          if(real(proby).gt.R) then
c count of unwgted events
             count(17) = count(17) + 1
    
c unweight that one event, then assign quark labels
c have all the part weights, normalise to total weight

             proby_dub = dsigma_dub/dsigma
             proby_scb = dsigma_scb/dsigma
             proby_dcb = dsigma_dcb/dsigma
             proby_sub = dsigma_sub/dsigma

             proby_ubd = dsigma_ubd/dsigma
             proby_cbs = dsigma_cbs/dsigma
             proby_cbd = dsigma_cbd/dsigma
             proby_ubs = dsigma_ubs/dsigma
             
             proby_udb = dsigma_udb/dsigma
             proby_csb = dsigma_csb/dsigma
             proby_usb = dsigma_usb/dsigma
             proby_cdb = dsigma_cdb/dsigma
             
             proby_dbu = dsigma_dbu/dsigma
             proby_sbc = dsigma_sbc/dsigma
             proby_sbu = dsigma_sbu/dsigma
             proby_dbc = dsigma_dbc/dsigma

c new random number
             R=rand(startseed)

             proby_tot=0.0

c W-
             proby_tot=proby_tot+proby_dub
             if(proby_tot.gt.R) then
                event = 1
                count(1) = count(1) + 1
                count(18) = count(18) + 1
c                write(*,*) proby_tot
                goto 200
             endif

             proby_tot=proby_tot+proby_scb
             if(proby_tot.gt.R) then
                event = 2
                count(2) = count(2) + 1
                count(18) = count(18) + 1
c                write(*,*) proby_tot
                goto 200
             endif

             proby_tot=proby_tot+proby_dcb
             if(proby_tot.gt.R) then
                event = 3
                count(3) = count(3) + 1
                count(18) = count(18) + 1
c                write(*,*) proby_tot
                goto 200
             endif

             proby_tot=proby_tot+proby_sub
             if(proby_tot.gt.R) then
                event = 4
                count(4) = count(4) + 1
                count(18) = count(18) + 1
c                write(*,*) proby_tot
                goto 200
             endif

             proby_tot=proby_tot+proby_ubd
             if(proby_tot.gt.R) then
                event = 5
                count(5) = count(5) + 1
                count(18) = count(18) + 1
c                write(*,*) proby_tot
                goto 200
             endif

             proby_tot=proby_tot+proby_cbs
             if(proby_tot.gt.R) then
                event = 6
                count(6) = count(6) + 1
                count(18) = count(18) + 1
c                write(*,*) proby_tot
                goto 200
             endif

             proby_tot=proby_tot+proby_cbd
             if(proby_tot.gt.R) then
                event = 7
                count(7) = count(7) + 1
                count(18) = count(18) + 1
c                write(*,*) proby_tot
                goto 200
             endif

             proby_tot=proby_tot+proby_ubs
             if(proby_tot.gt.R) then
                event = 8
                count(8) = count(8) + 1
                count(18) = count(18) + 1
c                write(*,*) proby_tot
                goto 200
             endif

c W+ !!! if we change order here, does it change results?
             proby_tot=proby_tot+proby_udb
             if(proby_tot.gt.R) then
                event = 9
                count(9) = count(9) + 1
                count(18) = count(18) + 1
c                write(*,*) proby_tot
                goto 200
             endif

             proby_tot=proby_tot+proby_csb
             if(proby_tot.gt.R) then
                event = 10
                count(10) = count(10) + 1
                count(18) = count(18) + 1
c                write(*,*) proby_tot
                goto 200
             endif

             proby_tot=proby_tot+proby_usb
             if(proby_tot.gt.R) then
                event = 11
                count(11) = count(11) + 1
                count(18) = count(18) + 1
c                write(*,*) proby_tot
                goto 200
             endif

             proby_tot=proby_tot+proby_cdb
             if(proby_tot.gt.R) then
                event = 12
                count(12) = count(12) + 1
                count(18) = count(18) + 1
c                write(*,*) proby_tot
                goto 200
             endif

             proby_tot=proby_tot+proby_dbu
             if(proby_tot.gt.R) then
                event = 13
                count(13) = count(13) + 1
                count(18) = count(18) + 1
c                write(*,*) proby_tot
                goto 200
             endif

             proby_tot=proby_tot+proby_sbc
             if(proby_tot.gt.R) then
                event = 14
                count(14) = count(14) + 1
                count(18) = count(18) + 1
c                write(*,*) proby_tot
                goto 200
             endif

             proby_tot=proby_tot+proby_sbu
             if(proby_tot.gt.R) then
                event = 15
                count(15) = count(15) + 1
                count(18) = count(18) + 1
c                write(*,*) proby_tot
                goto 200
             endif

             proby_tot=proby_tot+proby_dbc
             if(proby_tot.gt.R) then
                event = 16
                count(16) = count(16) + 1
                count(18) = count(18) + 1
c                write(*,*) proby_tot
                goto 200
             endif

c only unique events now

 200         call hf1(1025,h_eta_a1,1.)
             call hf1(1026,h_eta_a2,1.)
             call hf1(1027,h_eta_e,1.)
             call hf1(103,h_pt_a1,1.)
             call hf1(104,h_pt_a2,1.)
             call hf1(105,h_im_aa,1.)
             call hf1(106,h_dr_aa,1.)
             call hf1(1061,h_dr_ea1,1.)
             call hf1(1062,h_dr_ea2,1.)
             call hf1(107,h_phi_a1,1.)
             call hf1(108,h_phi_a2,1.)
             call hf1(109,h_cos_a1,1.)
             call hf1(1010,h_cos_a2,1.)
             call hf1(1011,h_m_w,1.)
             call hf1(1012,h_mt_w,1.)

c Calculate all possible W masses
c M(l,ni,gamma,gamma)
         MW1 = SQRT((p1(0)+p2(0)+p3(0)+p4(0))*(p1(0)+p2(0)+p3(0)+p4(0))-
     +            (p1(1)+p2(1)+p3(1)+p4(1))*(p1(1)+p2(1)+p3(1)+p4(1))-
     +            (p1(2)+p2(2)+p3(2)+p4(2))*(p1(2)+p2(2)+p3(2)+p4(2))-
     +            (p1(3)+p2(3)+p3(3)+p4(3))*(p1(3)+p2(3)+p3(3)+p4(3)))
c M(l,ni,gamma1)
             MW2 = SQRT((p1(0)+p3(0)+p4(0))*(p1(0)+p3(0)+p4(0))-
     +            (p1(1)+p3(1)+p4(1))*(p1(1)+p3(1)+p4(1))-
     +            (p1(2)+p3(2)+p4(2))*(p1(2)+p3(2)+p4(2))-
     +            (p1(3)+p3(3)+p4(3))*(p1(3)+p3(3)+p4(3)))
c M(l,ni,gamma2)
             MW3 = SQRT((p2(0)+p3(0)+p4(0))*(p2(0)+p3(0)+p4(0))-
     +      (p2(1)+p3(1)+p4(1))*(p2(1)+p3(1)+p4(1))-
     +      (p2(2)+p3(2)+p4(2))*(p2(2)+p3(2)+p4(2))-
     +      (p2(3)+p3(3)+p4(3))*(p2(3)+p3(3)+p4(3)))
c M(l,ni)
             MW4 = SQRT((p3(0)+p4(0))*(p3(0)+p4(0))-
     +      (p3(1)+p4(1))*(p3(1)+p4(1))-
     +      (p3(2)+p4(2))*(p3(2)+p4(2))-
     +      (p3(3)+p4(3))*(p3(3)+p4(3)))

c Type of event based on best fit to the W mass
             evtyp = MWMASSTYPE(MW1,MW2,MW3,MW4)
             write(20,*) "MWMASSTYPE = ", evtyp

c Label as ele/mu with 50/50 proby and set beta for max fixing
             elmurand = rand(startseed)
             write(20,*) "random = ", elmurand
             if(elmuflag .LT. elmurand) then
                lepidup = 11
                neutidup = 12
                beta = (elemass*elemass)/(MW4*MW4)  
             else
                lepidup = 13
                neutidup = 14      
                beta = (mumass*mumass)/(MW4*MW4)
             endif
             
c find LH info without W case
             if(wflag .EQ. 0) then 
                if(event.eq.1) then
                   IDUP(1)=1
                   IDUP(2)=-2
                   IDUP(3)=22
                   IDUP(4)=22
                   IDUP(5)=lepidup
                   IDUP(6)=-(neutidup)
                endif
                if(event.eq.2) then
                   IDUP(1)=3
                   IDUP(2)=-4
                   IDUP(3)=22
                   IDUP(4)=22
                   IDUP(5)=lepidup
                   IDUP(6)=-(neutidup)
                endif
                if(event.eq.3) then
                   IDUP(1)=1
                   IDUP(2)=-4
                   IDUP(3)=22
                   IDUP(4)=22
                   IDUP(5)=lepidup
                   IDUP(6)=-(neutidup)
                endif
                if(event.eq.4) then
                   IDUP(1)=3
                   IDUP(2)=-2
                   IDUP(3)=22
                   IDUP(4)=22
                   IDUP(5)=lepidup
                   IDUP(6)=-(neutidup)
                endif
                if(event.eq.5) then
                   IDUP(1)=-2
                   IDUP(2)=1
                   IDUP(3)=22
                   IDUP(4)=22
                   IDUP(5)=lepidup
                   IDUP(6)=-(neutidup)
                endif
                if(event.eq.6) then
                   IDUP(1)=-4
                   IDUP(2)=3
                   IDUP(3)=22
                   IDUP(4)=22
                   IDUP(5)=lepidup
                   IDUP(6)=-(neutidup)
                endif
                if(event.eq.7) then
                   IDUP(1)=-4
                   IDUP(2)=1
                   IDUP(3)=22
                   IDUP(4)=22
                   IDUP(5)=lepidup
                   IDUP(6)=-(neutidup)
                endif
                if(event.eq.8) then
                   IDUP(1)=-2
                   IDUP(2)=3
                   IDUP(3)=22
                   IDUP(4)=22
                   IDUP(5)=lepidup
                   IDUP(6)=-(neutidup)
                endif
                if(event.eq.9) then
                   IDUP(1)=2
                   IDUP(2)=-1
                   IDUP(3)=22
                   IDUP(4)=22
                   IDUP(5)=-(lepidup)
                   IDUP(6)=neutidup
                endif
                if(event.eq.10) then
                   IDUP(1)=4
                   IDUP(2)=-3
                   IDUP(3)=22
                   IDUP(4)=22
                   IDUP(5)=-(lepidup)
                   IDUP(6)=neutidup
                endif
                if(event.eq.11) then
                   IDUP(1)=2
                   IDUP(2)=-3
                   IDUP(3)=22
                   IDUP(4)=22
                   IDUP(5)=-(lepidup)
                   IDUP(6)=neutidup
                endif
                if(event.eq.12) then
                   IDUP(1)=4
                   IDUP(2)=-1
                   IDUP(3)=22
                   IDUP(4)=22
                   IDUP(5)=-(lepidup)
                   IDUP(6)=neutidup
                endif
                if(event.eq.13) then
                   IDUP(1)=-1
                   IDUP(2)=2
                   IDUP(3)=22
                   IDUP(4)=22
                   IDUP(5)=-(lepidup)
                   IDUP(6)=neutidup
                endif
                if(event.eq.14) then
                   IDUP(1)=-3
                   IDUP(2)=4
                   IDUP(3)=22
                   IDUP(4)=22
                   IDUP(5)=-(lepidup)
                   IDUP(6)=neutidup
                endif
                if(event.eq.15) then
                   IDUP(1)=-3
                   IDUP(2)=2
                   IDUP(3)=22
                   IDUP(4)=22
                   IDUP(5)=-(lepidup)
                   IDUP(6)=neutidup
                endif
                if(event.eq.16) then
                   IDUP(1)=-1
                   IDUP(2)=4
                   IDUP(3)=22
                   IDUP(4)=22
                   IDUP(5)=-(lepidup)
                   IDUP(6)=neutidup
                endif
                
                ISTUP(1)=-1
                ISTUP(2)=-1
                ISTUP(3)=1
                ISTUP(4)=1
                ISTUP(5)=1
                ISTUP(6)=1
         
                MOTHUP(1,1)=0
                MOTHUP(2,1)=0
                MOTHUP(1,2)=0
                MOTHUP(2,2)=0
                MOTHUP(1,3)=1
                MOTHUP(2,3)=2
                MOTHUP(1,4)=1
                MOTHUP(2,4)=2
                MOTHUP(1,5)=1
                MOTHUP(2,5)=2
                MOTHUP(1,6)=1
                MOTHUP(2,6)=2

c find LH info with W case
             else 

                if(event.eq.1) then
                   IDUP(1)=1
                   IDUP(2)=-2
                   IDUP(3)=-24
                   IDUP(4)=22
                   IDUP(5)=22
                   IDUP(6)=lepidup
                   IDUP(7)=-(neutidup)
                endif
                if(event.eq.2) then
                   IDUP(1)=3
                   IDUP(2)=-4
                   IDUP(3)=-24
                   IDUP(4)=22
                   IDUP(5)=22
                   IDUP(6)=lepidup
                   IDUP(7)=-(neutidup)
                endif
                if(event.eq.3) then
                   IDUP(1)=1
                   IDUP(2)=-4
                   IDUP(3)=-24
                   IDUP(4)=22
                   IDUP(5)=22
                   IDUP(6)=lepidup
                   IDUP(7)=-(neutidup)
                endif
                if(event.eq.4) then
                   IDUP(1)=3
                   IDUP(2)=-2
                   IDUP(3)=22
                   IDUP(4)=22
                   IDUP(5)=lepidup
                   IDUP(6)=-(neutidup)
                endif
                if(event.eq.5) then
                   IDUP(1)=-2
                   IDUP(2)=1
                   IDUP(3)=-24
                   IDUP(4)=22
                   IDUP(5)=22
                   IDUP(6)=lepidup
                   IDUP(7)=-(neutidup)
                endif
                if(event.eq.6) then
                   IDUP(1)=-4
                   IDUP(2)=3
                   IDUP(3)=-24
                   IDUP(4)=22
                   IDUP(5)=22
                   IDUP(6)=lepidup
                   IDUP(7)=-(neutidup)
                endif
                if(event.eq.7) then
                   IDUP(1)=-4
                   IDUP(2)=1
                   IDUP(3)=-24
                   IDUP(4)=22
                   IDUP(5)=22
                   IDUP(6)=lepidup
                   IDUP(7)=-(neutidup)
                endif
                if(event.eq.8) then
                   IDUP(1)=-2
                   IDUP(2)=3
                   IDUP(3)=-24
                   IDUP(4)=22
                   IDUP(5)=22
                   IDUP(6)=lepidup
                   IDUP(7)=-(neutidup)
                endif
                if(event.eq.9) then
                   IDUP(1)=2
                   IDUP(2)=-1
                   IDUP(3)=24
                   IDUP(4)=22
                   IDUP(5)=22
                   IDUP(6)=-(lepidup)
                   IDUP(7)=neutidup
                endif
                if(event.eq.10) then
                   IDUP(1)=4
                   IDUP(2)=-3
                   IDUP(3)=24
                   IDUP(4)=22
                   IDUP(5)=22
                   IDUP(6)=-(lepidup)
                   IDUP(7)=neutidup
                endif
                if(event.eq.11) then
                   IDUP(1)=2
                   IDUP(2)=-3
                   IDUP(3)=24
                   IDUP(4)=22
                   IDUP(5)=22
                   IDUP(6)=-(lepidup)
                   IDUP(7)=neutidup
                endif
                if(event.eq.12) then
                   IDUP(1)=4
                   IDUP(2)=-1
                   IDUP(3)=24
                   IDUP(4)=22
                   IDUP(5)=22
                   IDUP(6)=-(lepidup)
                   IDUP(7)=neutidup
                endif
                if(event.eq.13) then
                   IDUP(1)=-1
                   IDUP(2)=2
                   IDUP(3)=24
                   IDUP(4)=22
                   IDUP(5)=22
                   IDUP(6)=-(lepidup)
                   IDUP(7)=neutidup
                endif
                if(event.eq.14) then
                   IDUP(1)=-3
                   IDUP(2)=4
                   IDUP(3)=24
                   IDUP(4)=22
                   IDUP(5)=22
                   IDUP(6)=-(lepidup)
                   IDUP(7)=neutidup
                endif
                if(event.eq.15) then
                   IDUP(1)=-3
                   IDUP(2)=2
                   IDUP(3)=24
                   IDUP(4)=22
                   IDUP(5)=22
                   IDUP(6)=-(lepidup)
                   IDUP(7)=neutidup
                endif
                if(event.eq.16) then
                   IDUP(1)=-1
                   IDUP(2)=4
                   IDUP(3)=24
                   IDUP(4)=22
                   IDUP(5)=22
                   IDUP(6)=-(lepidup)
                   IDUP(7)=neutidup
                endif


                ISTUP(1)=-1
                ISTUP(2)=-1
                ISTUP(3)=2
                ISTUP(4)=1
                ISTUP(5)=1
                ISTUP(6)=1
                ISTUP(7)=1
                
c     with the w info we can set the parents properly
                if (evtyp .EQ. 1) then
                   MOTHUP(1,1)=0
                   MOTHUP(2,1)=0
                   MOTHUP(1,2)=0
                   MOTHUP(2,2)=0
                   MOTHUP(1,3)=1
                   MOTHUP(2,3)=2
                   MOTHUP(1,4)=3
                   MOTHUP(2,4)=3
                   MOTHUP(1,5)=3
                   MOTHUP(2,5)=3
                   MOTHUP(1,6)=3
                   MOTHUP(2,6)=3
                   MOTHUP(1,7)=3
                   MOTHUP(2,7)=3
                elseif (evtyp .EQ. 2) then
                   MOTHUP(1,1)=0
                   MOTHUP(2,1)=0
                   MOTHUP(1,2)=0
                   MOTHUP(2,2)=0
                   MOTHUP(1,3)=1
                   MOTHUP(2,3)=2
                   MOTHUP(1,4)=3
                   MOTHUP(2,4)=3
                   MOTHUP(1,5)=1
                   MOTHUP(2,5)=2
                   MOTHUP(1,6)=3
                   MOTHUP(2,6)=3
                   MOTHUP(1,7)=3
                   MOTHUP(2,7)=3
                elseif (evtyp .EQ. 3) then
                   MOTHUP(1,1)=0
                   MOTHUP(2,1)=0
                   MOTHUP(1,2)=0
                   MOTHUP(2,2)=0
                   MOTHUP(1,3)=1
                   MOTHUP(2,3)=2
                   MOTHUP(1,4)=1
                   MOTHUP(2,4)=2
                   MOTHUP(1,5)=3
                   MOTHUP(2,5)=3
                   MOTHUP(1,6)=3
                   MOTHUP(2,6)=3
                   MOTHUP(1,7)=3
                   MOTHUP(2,7)=3
                elseif (evtyp .EQ. 4) then
                   MOTHUP(1,1)=0
                   MOTHUP(2,1)=0
                   MOTHUP(1,2)=0
                   MOTHUP(2,2)=0
                   MOTHUP(1,3)=1
                   MOTHUP(2,3)=2
                   MOTHUP(1,4)=1
                   MOTHUP(2,4)=2
                   MOTHUP(1,5)=1
                   MOTHUP(2,5)=2
                   MOTHUP(1,6)=3
                   MOTHUP(2,6)=3
                   MOTHUP(1,7)=3
                   MOTHUP(2,7)=3
                endif
                
             endif
      
c   Write LH without W
             if(wflag .EQ. 0) then
         
                if(event.eq.1 .or. event.eq.2 .or. event.eq.3 .or.
     +               event.eq.4 .or. event.eq.9 .or. event.eq.10 .or.
     +               event.eq.11 .or. event.eq.12) then
                   ICOLUP(1,1)=500
                   ICOLUP(2,1)=0
                   ICOLUP(1,2)=0
                   ICOLUP(2,2)=500
                endif
                if(event.eq.5 .or. event.eq.6 .or. event.eq.7 .or.
     +               event.eq.8 .or. event.eq.13 .or. event.eq.14 .or.
     +               event.eq.15 .or. event.eq.16) then
                   ICOLUP(1,1)=0
                   ICOLUP(2,1)=501
                   ICOLUP(1,2)=501
                   ICOLUP(2,2)=0
                endif
                ICOLUP(1,3)=0
                ICOLUP(2,3)=0
                ICOLUP(1,4)=0
                ICOLUP(2,4)=0
                ICOLUP(1,5)=0
                ICOLUP(2,5)=0
                ICOLUP(1,6)=0
                ICOLUP(2,6)=0
         
                PUP(1,1)=pa(1)
                PUP(2,1)=pa(2)
                PUP(3,1)=pa(3)
                PUP(4,1)=pa(0)
                PUP(5,1)=0
                PUP(1,2)=pb(1)
                PUP(2,2)=pb(2)
                PUP(3,2)=pb(3)
                PUP(4,2)=pb(0)
                PUP(5,2)=0
                PUP(1,3)=p1(1)
                PUP(2,3)=p1(2)
                PUP(3,3)=p1(3)
                PUP(4,3)=p1(0)
                PUP(5,3)=0
                PUP(1,4)=p2(1)
                PUP(2,4)=p2(2)
                PUP(3,4)=p2(3)
                PUP(4,4)=p2(0)
                PUP(5,4)=0
                PUP(1,5)=p3(1)+beta*p4(1)
                PUP(2,5)=p3(2)+beta*p4(2)
                PUP(3,5)=p3(3)+beta*p4(3)
                PUP(4,5)=p3(0)+beta*p4(0)
                PUP(5,5)=sqrt(PUP(4,5)*PUP(4,5)-PUP(1,5)*PUP(1,5)-
     +               PUP(2,5)*PUP(2,5)-PUP(3,5)*PUP(3,5))
                PUP(1,6)=p4(1)*(1-beta)
                PUP(2,6)=p4(2)*(1-beta)
                PUP(3,6)=p4(3)*(1-beta)
                PUP(4,6)=p4(0)*(1-beta)
                PUP(5,6)=0
                
                VTIMUP(1)=-100
                VTIMUP(2)=-100
                VTIMUP(3)=-100
                VTIMUP(4)=-100
                VTIMUP(5)=-100
                VTIMUP(6)=-100
                
                SPINUP(1)=-100
                SPINUP(2)=-100
                SPINUP(3)=-100
                SPINUP(4)=-100
                SPINUP(5)=-100
                SPINUP(6)=-100
                
c Write LH with W
             else
                
                if(event.eq.1 .or. event.eq.2 .or. event.eq.3 .or.
     +               event.eq.4 .or. event.eq.9 .or. event.eq.10 .or.
     +               event.eq.11 .or. event.eq.12) then
                   ICOLUP(1,1)=500
                   ICOLUP(2,1)=0
                   ICOLUP(1,2)=0
                   ICOLUP(2,2)=500
                endif
                if(event.eq.5 .or. event.eq.6 .or. event.eq.7 .or.
     +               event.eq.8 .or. event.eq.13 .or. event.eq.14 .or.
     +               event.eq.15 .or. event.eq.16) then
                   ICOLUP(1,1)=0
                   ICOLUP(2,1)=501
                   ICOLUP(1,2)=501
                   ICOLUP(2,2)=0
                endif
                ICOLUP(1,3)=0
                ICOLUP(2,3)=0
                ICOLUP(1,4)=0
                ICOLUP(2,4)=0
                ICOLUP(1,5)=0
                ICOLUP(2,5)=0
                ICOLUP(1,6)=0
                ICOLUP(2,6)=0
                ICOLUP(1,7)=0
                ICOLUP(2,7)=0
         
                PUP(1,1)=pa(1)
                PUP(2,1)=pa(2)
                PUP(3,1)=pa(3)
                PUP(4,1)=pa(0)
                PUP(5,1)=0
                PUP(1,2)=pb(1)
                PUP(2,2)=pb(2)
                PUP(3,2)=pb(3)
                PUP(4,2)=pb(0)
                PUP(5,2)=0
                PUP(1,3)=0
                PUP(2,3)=0
                PUP(3,3)=0
                PUP(4,3)=0
                PUP(5,3)=0
                PUP(1,4)=p1(1)
                PUP(2,4)=p1(2)
                PUP(3,4)=p1(3)
                PUP(4,4)=p1(0)
                PUP(5,4)=0
                PUP(1,5)=p2(1)
                PUP(2,5)=p2(2)
                PUP(3,5)=p2(3)
                PUP(4,5)=p2(0)
                PUP(5,5)=0
                PUP(1,6)=p3(1)+beta*p4(1)
                PUP(2,6)=p3(2)+beta*p4(2)
                PUP(3,6)=p3(3)+beta*p4(3)
                PUP(4,6)=p3(0)+beta*p4(0)
                PUP(5,6)=sqrt(PUP(4,6)*PUP(4,6)-PUP(1,6)*PUP(1,6)-
     +               PUP(2,6)*PUP(2,6)-PUP(3,6)*PUP(3,6))
                PUP(1,7)=p4(1)*(1-beta)
                PUP(2,7)=p4(2)*(1-beta)
                PUP(3,7)=p4(3)*(1-beta)
                PUP(4,7)=p4(0)*(1-beta)
                PUP(5,7)=0
         
                VTIMUP(1)=-100
                VTIMUP(2)=-100
                VTIMUP(3)=-100
                VTIMUP(4)=-100
                VTIMUP(5)=-100
                VTIMUP(6)=-100
                VTIMUP(7)=-100
         
                SPINUP(1)=-100
                SPINUP(2)=-100
                SPINUP(3)=-100
                SPINUP(4)=-100
                SPINUP(5)=-100
                SPINUP(6)=-100
                SPINUP(7)=-100
                
             endif

c  write HEPEUP without W
             if(wflag .EQ. 0) then
                
              write(80,*) NUP, IDPRUP, XWGTUP, SCALUP, AQEDUP, AQCDUP
              write(80,*) IDUP(1), IDUP(2), IDUP(3), IDUP(4),
     +           IDUP(5), IDUP(6)
              write(80,*) ISTUP(1), ISTUP(2), ISTUP(3), ISTUP(4),
     +           ISTUP(5), ISTUP(6)
              write(80,*) 
     +           MOTHUP(1,1),MOTHUP(2,1),MOTHUP(1,2),MOTHUP(2,2),
     +           MOTHUP(1,3),MOTHUP(2,3),MOTHUP(1,4),MOTHUP(2,4),         
     +           MOTHUP(1,5),MOTHUP(2,5),MOTHUP(1,6),MOTHUP(2,6)
              write(80,*) 
     +           ICOLUP(1,1),ICOLUP(2,1),ICOLUP(1,2),ICOLUP(2,2),
     +           ICOLUP(1,3),ICOLUP(2,3),ICOLUP(1,4),ICOLUP(2,4),
     +           ICOLUP(1,5),ICOLUP(2,5),ICOLUP(1,6),ICOLUP(2,6)
              write(80,*) 
     +           PUP(1,1),PUP(2,1),PUP(3,1),PUP(4,1),PUP(5,1),
     +           PUP(1,2),PUP(2,2),PUP(3,2),PUP(4,2),PUP(5,2),
     +           PUP(1,3),PUP(2,3),PUP(3,3),PUP(4,3),PUP(5,3),
     +           PUP(1,4),PUP(2,4),PUP(3,4),PUP(4,4),PUP(5,4),
     +           PUP(1,5),PUP(2,5),PUP(3,5),PUP(4,5),PUP(5,5),
     +           PUP(1,6),PUP(2,6),PUP(3,6),PUP(4,6),PUP(5,6)
             write(80,*) 
     +           VTIMUP(1), VTIMUP(2), VTIMUP(3), VTIMUP(4),
     +           VTIMUP(5), VTIMUP(6)
             write(80,*) 
     +           SPINUP(1), SPINUP(2), SPINUP(3), SPINUP(4),
     +           SPINUP(5), SPINUP(6)
         

c write HEPEUP with W
            else
            
             write(80,*) NUP, IDPRUP, XWGTUP, SCALUP, AQEDUP, AQCDUP
             write(80,*) 
     +           IDUP(1), IDUP(2), IDUP(3), IDUP(4),
     +           IDUP(5), IDUP(6), IDUP(7)
             write(80,*) 
     +           ISTUP(1), ISTUP(2), ISTUP(3), ISTUP(4),
     +           ISTUP(5), ISTUP(6), ISTUP(7)
             write(80,*) 
     +           MOTHUP(1,1),MOTHUP(2,1),MOTHUP(1,2),MOTHUP(2,2),
     +           MOTHUP(1,3),MOTHUP(2,3),MOTHUP(1,4),MOTHUP(2,4),
     +           MOTHUP(1,5),MOTHUP(2,5),MOTHUP(1,6),MOTHUP(2,6),
     +           MOTHUP(1,7),MOTHUP(2,7)
             write(80,*) 
     +           ICOLUP(1,1),ICOLUP(2,1),ICOLUP(1,2),ICOLUP(2,2),
     +           ICOLUP(1,3),ICOLUP(2,3),ICOLUP(1,4),ICOLUP(2,4),
     +           ICOLUP(1,5),ICOLUP(2,5),ICOLUP(1,6),ICOLUP(2,6),
     +           ICOLUP(1,7),ICOLUP(2,7)
             write(80,*) 
     +           PUP(1,1),PUP(2,1),PUP(3,1),PUP(4,1),PUP(5,1),
     +           PUP(1,2),PUP(2,2),PUP(3,2),PUP(4,2),PUP(5,2),
     +           PUP(1,3),PUP(2,3),PUP(3,3),PUP(4,3),PUP(5,3),
     +           PUP(1,4),PUP(2,4),PUP(3,4),PUP(4,4),PUP(5,4),
     +           PUP(1,5),PUP(2,5),PUP(3,5),PUP(4,5),PUP(5,5),
     +           PUP(1,6),PUP(2,6),PUP(3,6),PUP(4,6),PUP(5,6),
     +           PUP(1,7),PUP(2,7),PUP(3,7),PUP(4,7),PUP(5,7)
             write(80,*) 
     +           VTIMUP(1), VTIMUP(2), VTIMUP(3), VTIMUP(4),
     +           VTIMUP(5), VTIMUP(6), VTIMUP(7)
             write(80,*) 
     +           SPINUP(1), SPINUP(2), SPINUP(3), SPINUP(4),
     +           SPINUP(5), SPINUP(6), SPINUP(7)
        
            endif
         
         endif

      endif
      
      return
      end 


*******************************************
*                   Scalar Product A.B
*******************************************
*
       FUNCTION dot_d(A,B)
       real*8 A(0:3), B(0:3), dot_d
       dot_d = A(0)*B(0) - A(1)*B(1) - A(2)*B(2) - A(3)*B(3)
       RETURN
       END

c Function for searchig of the mass closest to W mass
      integer function MWMASSTYPE(MW1,MW2,MW3,MW4)
      double precision MW1, MW2, MW3, MW4
      double precision MW1M80, MW2M80, MW3M80, MW4M80

      MW1M80 = abs(80.4-MW1)
      MW2M80 = abs(80.4-MW2)
      MW3M80 = abs(80.4-MW3)
      MW4M80 = abs(80.4-MW4)

      write(20,*) MW1M80, MW2M80, MW3M80, MW4M80

      if (MW1M80 .LT. MW2M80 .AND. MW1M80 .LT. MW3M80 .AND.
     +    MW1M80 .LT. MW4M80) then
             MWMASSTYPE=1
      endif
      if (MW2M80 .LT. MW1M80 .AND. MW2M80 .LT. MW3M80 .AND.
     +    MW2M80 .LT. MW4M80) then
             MWMASSTYPE=2
      endif
      if (MW3M80 .LT. MW1M80 .AND. MW3M80 .LT. MW2M80 .AND.
     +    MW3M80 .LT. MW4M80) then
             MWMASSTYPE=3
      endif
      if (MW4M80 .LT. MW1M80 .AND. MW4M80 .LT. MW2M80 .AND.
     +    MW4M80 .LT. MW3M80) then
             MWMASSTYPE=4
      endif


      return
      end
