c
c ----------------------------------------------------------------------
c
      subroutine wmwpza_ano(wm,wp,z,a,gano, vertex)
c
c this subroutine computes an amplitude of the four-point coupling of   
c the w-, w+ and two a.  
c                                                                       
c input:                                                                
c       complex wm(0:3)        : flow-out w-                         wm 
c       complex wp(0:3)        : flow-out w+                         wp 
c       complex z(0:3)         : first    z                          z
c       complex a(0:3)         : second   a                          a 
c       real    gano(11)       : anomalous coupling constant  
c                                                   gano( 1) = beta_W0a 
c                                                   gano( 2) = beta_Wca        
c                                                   gano( 3) = beta_W0Z 
c                                                   gano( 4) = beta_WcZ       
c                                                   gano( 5) = beta_W1Z       
c                                                   gano( 6) = beta_W2Z 
c                                                   gano( 7) = beta_W3Z    
c                                                   gano( 8) = beta_Z0a 
c                                                   gano( 9) = beta_Zca        
c                                                   gano(10) = beta_Z0Z 
c                                                   gano(11) = beta_ZcZ    
c where all the bosons are defined by the flowing-out quantum number.   
c                                                                       
c output:                                                               
c       complex vertex         : amplitude          gamma(wm,wp,z,a)
c
      complex*16    wm(6),wp(6),z(6),a(6),vertex
      real*8     pwm(0:3),pwp(0:3),pz(0:3),pa(0:3)
      real*8 rdotpp
      complex*16 cdotvp,cdotvv
      real*8     gano(11)
c      
      double precision  gw, gwwa, gwwz
      common /coup1/    gw, gwwa, gwwz
      real*8 r_zero, r_one , r_four, r_ote, r_pi, r_ialph
      parameter( r_zero=0.0d0, r_one=1.0d0 )
      parameter( r_four=4.0d0, r_ote=128.0d0 )
      parameter( r_pi=3.14159265358979323846d0, r_ialph=137.0359895d0 )
c
      real*8    alpha,fourpi,ee,ee2,sw,sw2,cw,cw2
c
c      alpha = r_one / r_ialph
      alpha = r_one / r_ote
      fourpi = r_four * r_pi
      ee2=alpha * fourpi
      ee=sqrt( alpha * fourpi )
      sw    =  ee/gw
      sw2   =  sw*sw
      cw2   =  1.d0 - sw2
      cw    =  dsqrt( cw2 ) 
c
      pwm(0)=dble( wm(5))
      pwm(1)=dble( wm(6))
      pwm(2)=dimag(wm(6))
      pwm(3)=dimag(wm(5))
      pwp(0)=dble( wp(5))
      pwp(1)=dble( wp(6))
      pwp(2)=dimag(wp(6))
      pwp(3)=dimag(wp(5))
      pz(0)=dble( z(5))
      pz(1)=dble( z(6))
      pz(2)=dimag(z(6))
      pz(3)=dimag(z(5))
      pa(0)=dble( a(5))
      pa(1)=dble( a(6))
      pa(2)=dimag(a(6))
      pa(3)=dimag(a(5))
c
         vertex = (ee2/2.d0)*gano(3)*
     &            (cdotvv(wm,wp)*cdotvp(a,pz)*cdotvp(z,pa)
     &            -cdotvv(wm,wp)*rdotpp(pz,pa)*cdotvv(z,a))
     &          + (ee2/8.d0)*gano(4)*
     &            (-rdotpp(pz,pa)*cdotvv(z,wm)*cdotvv(a,wp)
     &             -rdotpp(pz,pa)*cdotvv(z,wp)*cdotvv(a,wm)
     &             -cdotvv(z,a)*cdotvp(wm,pz)*cdotvp(wp,pa)
     &             -cdotvv(z,a)*cdotvp(wp,pz)*cdotvp(wm,pa)
     &             +cdotvp(z,pa)*cdotvp(wm,pz)*cdotvv(a,wp)
     &             +cdotvp(z,pa)*cdotvp(wp,pz)*cdotvv(a,wm)
     &             +cdotvp(a,pz)*cdotvp(wp,pa)*cdotvv(z,wm)
     &             +cdotvp(a,pz)*cdotvp(wm,pa)*cdotvv(z,wp))
     &          + (ee2/(4.d0*cw*sw))*gano(5)*
     &            (+cdotvp(wp,pa)*cdotvp(a,pwp)*cdotvv(z,wm)
     &             +cdotvp(wm,pa)*cdotvp(a,pwm)*cdotvv(z,wp)
     &             -rdotpp(pwp,pa)*cdotvv(wp,a)*cdotvv(wm,z)
     &             -rdotpp(pwm,pa)*cdotvv(wm,a)*cdotvv(wp,z))
     &          + (ee2/(8.d0*cw*sw))*gano(6)*
     &            (+cdotvp(z,pa)*cdotvp(a,pwp)*cdotvv(wp,wm)
     &             -cdotvp(z,pa)*cdotvp(wm,pwp)*cdotvv(wp,a)
     &             +cdotvp(z,pa)*cdotvp(a,pwm)*cdotvv(wp,wm)
     &             -cdotvp(z,pa)*cdotvp(wp,pwm)*cdotvv(wm,a)
     &             -rdotpp(pwp,pa)*cdotvv(wp,wm)*cdotvv(a,z)
     &             +cdotvp(wp,pa)*cdotvp(wm,pwp)*cdotvv(a,z)
     &             -rdotpp(pwm,pa)*cdotvv(wp,wm)*cdotvv(a,z)
     &             +cdotvp(wm,pa)*cdotvp(wp,pwm)*cdotvv(a,z))
     &          + (ee2/(8.d0*cw*sw))*gano(7)*
     &            (+cdotvp(wm,pa)*cdotvp(a,pwp)*cdotvv(wp,z)  
     &             -cdotvp(wm,pa)*cdotvp(z,pwp)*cdotvv(a,wp)  
     &             +cdotvp(wp,pa)*cdotvp(a,pwm)*cdotvv(wm,z) 
     &             -cdotvp(wp,pa)*cdotvp(z,pwm)*cdotvv(a,wm)  
     &             -rdotpp(pa,pwp)*cdotvv(a,wm)*cdotvv(z,wp)   
     &             +cdotvp(wp,pa)*cdotvp(z,pwp)*cdotvv(a,wm)
     &             -rdotpp(pa,pwm)*cdotvv(a,wp)*cdotvv(z,wm)   
     &             +cdotvp(wm,pa)*cdotvp(z,pwm)*cdotvv(a,wp))  
      return
      end


