c
c ----------------------------------------------------------------------
c
      subroutine zzaaxx_ano(z1,z2,a1,a2,gano,vertex)
c
c this subroutine computes an amplitude of the four-point coupling of   
c two z and two a.  
c                                                                       
c input:                                                                
c       complex z1(0:3)        : flow-out z                          z1 
c       complex z2(0:3)        : flow-out z                          z2 
c       complex a1(0:3)        : first    a                          a1
c       complex a2(0:3)        : second   a                          a2
c       real    gano(11)       : anomalous coupling constant  
c                                                   gano( 1) = beta_W0a 
c                                                   gano( 2) = beta_Wca        
c                                                   gano( 3) = beta_W0Z 
c                                                   gano( 4) = beta_WcZ       
c                                                   gano( 5) = beta_W1Z       
c                                                   gano( 6) = beta_W2Z 
c                                                   gano( 7) = beta_W3Z    
c                                                   gano( 8) = beta_Z0a 
c                                                   gano( 9) = beta_Zca        
c                                                   gano(10) = beta_Z0Z 
c                                                   gano(11) = beta_ZcZ     
c where all the bosons are defined by the flowing-out quantum number.   
c                                                                       
c output:                                                               
c       complex vertex         : amplitude          gamma(z1,z2,a1,a2)
c
      complex*16    z1(6),z2(6),a1(6),a2(6),vertex
      real*8     pz1(0:3),pz2(0:3),pa1(0:3),pa2(0:3)
      real*8 rdotpp
      complex*16 cdotvp,cdotvv
      real*8     gano(11)
c      
      double precision  gw, gwwa, gwwz
      common /coup1/    gw, gwwa, gwwz 
      real*8 r_zero, r_one , r_four, r_ote, r_pi, r_ialph
      parameter( r_zero=0.0d0, r_one=1.0d0 )
      parameter( r_four=4.0d0, r_ote=128.0d0 )
      parameter( r_pi=3.14159265358979323846d0, r_ialph=137.0359895d0 )
c
      real*8    alpha,fourpi,ee,ee2,sw,sw2,cw,cw2
c
c      alpha = r_one / r_ialph
      alpha = r_one / r_ote
      fourpi = r_four * r_pi
      ee2=alpha * fourpi
      ee=sqrt( alpha * fourpi )
      sw    =  ee/gw
      sw2   =  sw*sw
      cw2   =  1.d0 - sw2
      cw    =  dsqrt( cw2 ) 
c
      pz1(0)=dble( z1(5))
      pz1(1)=dble( z1(6))
      pz1(2)=dimag(z1(6))
      pz1(3)=dimag(z1(5))
      pz2(0)=dble( z2(5))
      pz2(1)=dble( z2(6))
      pz2(2)=dimag(z2(6))
      pz2(3)=dimag(z2(5))
      pa1(0)=dble( a1(5))
      pa1(1)=dble( a1(6))
      pa1(2)=dimag(a1(6))
      pa1(3)=dimag(a1(5))
      pa2(0)=dble( a2(5))
      pa2(1)=dble( a2(6))
      pa2(2)=dimag(a2(6))
      pa2(3)=dimag(a2(5))
c
         vertex = (ee2/(4.d0*cw2))*gano(8)*
     &            (2.d0*cdotvv(z1,z2)*cdotvp(a2,pa1)*cdotvp(a1,pa2)
     &            -2.d0*cdotvv(z1,z2)*rdotpp(pa1,pa2)*cdotvv(a1,a2))
     &          + (ee2/(16.d0*cw2))*gano(9)*
     &            (-2.d0*rdotpp(pa1,pa2)*cdotvv(a1,z1)*cdotvv(a2,z2)
     &             -2.d0*rdotpp(pa1,pa2)*cdotvv(a1,z2)*cdotvv(a2,z1)
     &             -2.d0*cdotvv(a1,a2)*cdotvp(z1,pa1)*cdotvp(z2,pa2)
     &             -2.d0*cdotvv(a1,a2)*cdotvp(z2,pa1)*cdotvp(z1,pa2)
     &             +2.d0*cdotvp(a2,pa1)*cdotvp(z2,pa2)*cdotvv(a1,z1)
     &             +2.d0*cdotvp(a2,pa1)*cdotvp(z1,pa2)*cdotvv(a1,z2)
     &             +2.d0*cdotvp(a1,pa2)*cdotvp(z1,pa1)*cdotvv(a2,z2)
     &             +2.d0*cdotvp(a1,pa2)*cdotvp(z2,pa1)*cdotvv(a2,z1))
c
      return
      end
