c
c ----------------------------------------------------------------------
c
      subroutine zzzaxx_ano(z1,z2,z3,a,gano,vertex)
c
c this subroutine computes an amplitude of the four-point coupling of   
c two z and two a.  
c                                                                       
c input:                                                                
c       complex z1(0:3)        : flow-out z                          z1 
c       complex z2(0:3)        : flow-out z                          z2 
c       complex z3(0:3)        : third    z                          z3
c       complex a(0:3)         : first    a                          a 
c       real    gano(11)       : anomalous coupling constant
c                                                   gano( 1) = beta_W0a
c                                                   gano( 2) = beta_Wca
c                                                   gano( 3) = beta_W0Z
c                                                   gano( 4) = beta_WcZ
c                                                   gano( 5) = beta_W1Z
c                                                   gano( 6) = beta_W2Z
c                                                   gano( 7) = beta_W3Z
c                                                   gano( 8) = beta_Z0a
c                                                   gano( 9) = beta_Zca
c                                                   gano(10) = beta_Z0Z
c                                                   gano(11) = beta_ZcZ     
c where all the bosons are defined by the flowing-out quantum number.   
c                                                                       
c output:                                                               
c       complex vertex         : amplitude          gamma(z1,z2,z3,a)
c
      complex*16    z1(6),z2(6),z3(6),a(6),vertex
      real*8     pz1(0:3),pz2(0:3),pz3(0:3),pa(0:3)
      real*8 rdotpp
      complex*16 cdotvp,cdotvv
      real*8     gano(11)
c      
      double precision  gw, gwwa, gwwz
      common /coup1/    gw, gwwa, gwwz 
      real*8 r_zero, r_one , r_four, r_ote, r_pi, r_ialph
      parameter( r_zero=0.0d0, r_one=1.0d0 )
      parameter( r_four=4.0d0, r_ote=128.0d0 )
      parameter( r_pi=3.14159265358979323846d0, r_ialph=137.0359895d0 )
c
      real*8    alpha,fourpi,ee,ee2,sw,sw2,cw,cw2
c
c      alpha = r_one / r_ialph
      alpha = r_one / r_ote
      fourpi = r_four * r_pi
      ee2=alpha * fourpi
      ee=sqrt( alpha * fourpi )
      sw    =  ee/gw
      sw2   =  sw*sw
      cw2   =  1.d0 - sw2
      cw    =  dsqrt( cw2 ) 
c
      pz1(0)=dble( z1(5))
      pz1(1)=dble( z1(6))
      pz1(2)=dimag(z1(6))
      pz1(3)=dimag(z1(5))
      pz2(0)=dble( z2(5))
      pz2(1)=dble( z2(6))
      pz2(2)=dimag(z2(6))
      pz2(3)=dimag(z2(5))
      pz3(0)=dble( z3(5))
      pz3(1)=dble( z3(6))
      pz3(2)=dimag(z3(6))
      pz3(3)=dimag(z3(5))
      pa(0)=dble( a(5))
      pa(1)=dble( a(6))
      pa(2)=dimag(a(6))
      pa(3)=dimag(a(5))
cc
         vertex = (ee2/(4.d0*cw2))*gano(10)*
     &            (-2.d0*rdotpp(pa,pz1)*cdotvv(a,z1)*cdotvv(z2,z3)
     &             -2.d0*rdotpp(pa,pz2)*cdotvv(a,z2)*cdotvv(z1,z3)
     &             -2.d0*rdotpp(pa,pz3)*cdotvv(a,z3)*cdotvv(z1,z2)
     &             +2.d0*cdotvp(a,pz1)*cdotvp(z1,pa)*cdotvv(z2,z3)
     &             +2.d0*cdotvp(a,pz2)*cdotvp(z2,pa)*cdotvv(z1,z3)
     &             +2.d0*cdotvp(a,pz3)*cdotvp(z3,pa)*cdotvv(z1,z2))
     &          + (ee2/(8.d0*cw2))*gano(11)*
     &            (-cdotvv(a,z2)*cdotvv(z1,z3)*rdotpp(pa,pz1)
     &             -cdotvv(a,z3)*cdotvv(z1,z2)*rdotpp(pa,pz1)
     &             -cdotvv(a,z1)*cdotvv(z2,z3)*rdotpp(pa,pz2)
     &             -cdotvv(a,z3)*cdotvv(z1,z2)*rdotpp(pa,pz2)
     &             -cdotvv(a,z2)*cdotvv(z1,z3)*rdotpp(pa,pz3)
     &             -cdotvv(a,z1)*cdotvv(z2,z3)*rdotpp(pa,pz3)
     &             -cdotvv(a,z1)*cdotvp(z3,pz1)*cdotvp(z2,pa)
     &             -cdotvv(a,z1)*cdotvp(z2,pz1)*cdotvp(z3,pa)
     &             -cdotvv(a,z2)*cdotvp(z3,pz2)*cdotvp(z1,pa)
     &             -cdotvv(a,z2)*cdotvp(z1,pz2)*cdotvp(z3,pa)
     &             -cdotvv(a,z3)*cdotvp(z1,pz3)*cdotvp(z2,pa)
     &             -cdotvv(a,z3)*cdotvp(z2,pz3)*cdotvp(z1,pa)
     &             +cdotvv(z1,z3)*cdotvp(a,pz1)*cdotvp(z2,pa)
     &             +cdotvv(a,z2)*cdotvp(z3,pz1)*cdotvp(z1,pa) 
     &             +cdotvv(a,z3)*cdotvp(z2,pz1)*cdotvp(z1,pa)
     &             +cdotvv(z1,z2)*cdotvp(a,pz1)*cdotvp(z3,pa)
     &             +cdotvv(z2,z3)*cdotvp(a,pz2)*cdotvp(z1,pa)
     &             +cdotvv(a,z1)*cdotvp(z3,pz2)*cdotvp(z2,pa)
     &             +cdotvv(a,z3)*cdotvp(z1,pz2)*cdotvp(z2,pa)
     &             +cdotvv(z1,z2)*cdotvp(a,pz2)*cdotvp(z3,pa)
     &             +cdotvv(z1,z3)*cdotvp(a,pz3)*cdotvp(z2,pa)
     &             +cdotvv(a,z2)*cdotvp(z1,pz3)*cdotvp(z3,pa)
     &             +cdotvv(a,z1)*cdotvp(z2,pz3)*cdotvp(z3,pa)
     &             +cdotvv(z2,z3)*cdotvp(a,pz3)*cdotvp(z1,pa))
c
      return
      end
