cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
	subroutine phase_4(x,m12max,m12min,m34max,m34min,
     %                     plab,p1,p2,p3,p4)   
	
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c it generates the phase space for 4 particles in the final 
c state using as variables the invariant mass of the pairs (12) (34)
c the values m12max m12min and m34max and m34min are the ranges of the
c invariant masses to integrate. It is suitable for processes of two
c particles decaying into two. 
c
c plab is the momentum of the cms of the 4 particles in the lab frame (input)
c 
c p1,p2,p3,p4 are the generated momentum of the 4 paricles in the lab frame
c
c x(1 -> 8) is the vector with the random generated variables
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
	implicit none
c
	real*8 x(10)
	real*8 m12max,m12min,m34max,m34min
	real*8 m12,m34
	real*8 m12sq,m34sq
	real*8 m1sq,m2sq
	real*8 m3sq,m4sq
c
	real*8 plab(0:3)
	real*8 p1(0:3),p2(0:3),p3(0:3),p4(0:3)
	real*8 p12(0:3),p34(0:3)
	real*8 pb34(0:3),pb12(0:3)
	real*8 p1_12(0:3),p2_12(0:3),p3_34(0:3),p4_34(0:3)
        real*8 p1cm(0:3),p2cm(0:3),p3cm(0:3),p4cm(0:3)
        real*8 pm12sq,pm34sq,pm12,pm34
c
	real*8 cost,cos12,cos34 
	real*8 sint,sin12,sin34
	real*8 phi,phi12,phi34
	real*8 theta,theta12,theta34
	real*8 theta12_min,theta12_max
	real*8 theta34_min,theta34_max
c
	real*8 E12,E34 
	real*8 jac1,jac2,jac3
c
	real*8 pm1,pm3,lambda,pi,shat,root_s
c
	real*8         jacobian, flux 
        common/momenta/jacobian, flux  
c    
        real*8            m12r,w12r,m34r,w34r
        common/resonances/m12r,w12r,m34r,w34r
c
	real*8      m1,m2,m3,m4
        common/mass/m1,m2,m3,m4
c
	integer i
c
        pi=3.14159265359d0 
        shat=plab(0)*plab(0)-plab(1)*plab(1)-plab(2)*plab(2)
     %                      -plab(3)*plab(3)
        root_s=dsqrt(shat)

c Evaluating the flux
        flux=2.d0*shat      
        if(flux.eq.0.d0) write(*,*)'problem: flux=',flux
        
c First I decay the momentum plab into two "particles" with 
c in variant masses M12 and M34
        phi=2.d0*pi*x(1)
        cost=-1.d0+2.d0*x(2)
        sint=dsqrt(1.d0-cost*cost) 
        
c To help the code to find the resonances I use these variables

        m12=m12min+(m12max-m12min)*x(3)
        m12sq=m12*m12
        
        m34max=min(m34max,root_s-m12)
        m34=m34min+(m34max-m34min)*x(4)
        m34sq=m34*m34

        pm12sq=lambda(shat,m12sq,m34sq)/4.d0/shat
        E12=dsqrt(pm12sq+m12sq)
        pm12=dsqrt(pm12sq)
        pm34=pm12  
        E34=dsqrt(pm12sq+m34sq)
        
        p12(0)=E12
        p12(1)=pm12*sint*cos(phi)
        p12(2)=pm12*sint*sin(phi)
        p12(3)=pm12*cost
        
        p34(0)=E34
        p34(1)=-p12(1)
        p34(2)=-p12(2)
        p34(3)=-p12(3)
        
        jac1=(pm12/root_s)/(2.d0*(2.d0*pi)**5)
        
c Now we decay in the rest frame of the 12 particles    
        
        phi12=2.d0*pi*x(5)
        cos12=-1.d0+2.d0*x(6)
        sin12=dsqrt(1.d0-cos12*cos12)  
        m1sq=m1*m1
        m2sq=m2*m2
        pm1=dsqrt(lambda(m12sq,m1sq,m2sq))/2.d0/m12
        p1_12(0)=(m12sq+m1sq-m2sq)/2.d0/m12
        p1_12(1)=pm1*sin12*cos(phi12)        
        p1_12(2)=pm1*sin12*sin(phi12)
        p1_12(3)=pm1*cos12
   
        p2_12(0)=(m12sq-m1sq+m2sq)/2.d0/m12       
        p2_12(1)=-p1_12(1) 
        p2_12(2)=-p1_12(2) 
        p2_12(3)=-p1_12(3) 

        jac2=(pm1/m12)/(2.d0*(2.d0*pi)**5)       
                 
c Now we decay in the rest frame of the 34 particles    
        
        phi34=2.d0*pi*x(7)
        cos34=-1.d0+2.d0*x(8)
        sin34=dsqrt(1.d0-cos34*cos34)  
        m3sq=m3*m3
        m4sq=m4*m4
        pm3=dsqrt(lambda(m34sq,m3sq,m4sq))/2.d0/m34
        p3_34(0)=(m34sq+m3sq-m4sq)/2.d0/m34
        p3_34(1)=pm3*sin34*cos(phi34)
        p3_34(2)=pm3*sin34*sin(phi34)
        p3_34(3)=pm3*cos34
   
        p4_34(0)=(m34sq-m3sq+m4sq)/2.d0/m34
        p4_34(1)=-p3_34(1) 
        p4_34(2)=-p3_34(2) 
        p4_34(3)=-p3_34(3) 
       
c	write(*,*) p4_34 
c	write(*,*) p3_34 

        jac3=(pm3/m34)/(2.d0*(2.d0*pi)**5)
           
c Boost to the lab frame       
        do i=0,3       
        pb34(i)=plab(i)+p34(i)
        pb12(i)=plab(i)+p12(i)       
        end do
        
c	write(*,*) p12
c	write(*,*) p4_34 
c	write(*,*) p3_34 

        call boost(p12,p1_12,p2_12,p1cm,p2cm)
        call boost(p34,p3_34,p4_34,p3cm,p4cm)
        call boost(plab,p1cm,p2cm,p1,p2)
        call boost(plab,p3cm,p4cm,p3,p4)

c Evaluating the jacobian
        jacobian=jac1*jac2*jac3*(2.d0*pi)**10*
     %   (m12max-m12min)*(m34max-m34min)*4.d0*m12*m34
      
        if(jacobian.le.0.d0) write(*,*)'problem: jacobian=',jacobian

        return         
        end

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
       subroutine boost(pb,p1,p2,p1b,p2b)
c      it boosts the momentums p1b and p2b from their cm reference frame
c      to the lab frame. pb is the momentum of the p1b,p2b cm ref frame
c      in the lab frame.
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
       	
       implicit none
       integer i, j, n
       real*8 bg1, g1, bg2, g2, bg3, g3,pb(0:3),p1(0:3),p2(0:3)
       real*8 p1b(0:3),p2b(0:3),m
       
       m=dsqrt(pb(0)*pb(0)-pb(1)*pb(1)-pb(2)*pb(2)-pb(3)*pb(3))
       bg1 = pb(1)/m
       g1  = dsqrt(1.d0 + bg1**2)
       bg2 = pb(2) /m /g1
       g2  = dsqrt(1.d0 + bg2**2)
       bg3 = pb(3) /m /g1 /g2
       g3  = dsqrt(1.d0 + bg3**2)

       p1b(0)  = g1*g2*g3*p1(0) + bg1*g2*g3*p1(1) 
     %     + bg2*g3*p1(2) + bg3*p1(3)
       p1b(1) = bg1*p1(0) + g1*p1(1)
       p1b(2) = bg2*g1*p1(0) + bg2*bg1*p1(1) + g2*p1(2)
       p1b(3) = bg3*g2*g1*p1(0) + bg3*g2*bg1*p1(1)
     %      + bg3*bg2*p1(2) + g3*p1(3)

       p2b(0)  = g1*g2*g3*p2(0) + bg1*g2*g3*p2(1) 
     %     + bg2*g3*p2(2) + bg3*p2(3)
       p2b(1) = bg1*p2(0) + g1*p2(1)
       p2b(2) = bg2*g1*p2(0) + bg2*bg1*p2(1) + g2*p2(2)
       p2b(3) = bg3*g2*g1*p2(0) + bg3*g2*bg1*p2(1)
     %      + bg3*bg2*p2(2) + g3*p2(3)

         return
         end

c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

      function lambda(a, b, c)

      implicit none
      real*8 a, b, c, lambda

      lambda = a**2 + b**2 + c**2 - 2.d0*a*b - 2.d0*b*c - 2.d0*c*a

      return
      end


